@@ -1,30 +1,33 @@
 use std::{any::TypeId, collections::HashMap, marker::PhantomData, pin::Pin, sync::Arc};
 
 use futures::{Future, FutureExt};
-use receiver::ReceiverTrait;
 use tokio::sync::Mutex;
 
 use crate::{
-    receiver::{self, Receiver},
+    receiver::{Receiver, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
     Bus, BusInner, Message, Untyped,
 };
 
-pub trait ReceiverSubscriber<T: 'static> {
-    fn subscribe(
-        self,
+pub trait ReceiverSubscriberBuilder<T, M, R, E>:
+    SendUntypedReceiver + SendTypedReceiver<M> + ReciveTypedReceiver<R, E>
+where
+    T: 'static,
+    M: Message,
+    R: Message,
+    E: crate::Error,
+{
+    type Config: Default;
+
+    fn build(
+        cfg: Self::Config,
     ) -> (
-        Arc<dyn ReceiverTrait>,
+        Self,
         Box<
             dyn FnOnce(Untyped) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
         >,
-    );
-}
-
-pub trait ReceiverSubscriberBuilder<M, T: 'static> {
-    type Entry: ReceiverSubscriber<T>;
-    type Config: Default;
-
-    fn build(cfg: Self::Config) -> Self::Entry;
+    )
+    where
+        Self: Sized;
 }
 
 pub struct SyncEntry;
@@ -44,6 +47,7 @@ pub struct RegisterEntry<K, T> {
                 )
                     -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
             >,
+            Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
         )>,
     >,
     _m: PhantomData<(K, T)>,
@@ -54,10 +58,10 @@ impl<K, T: 'static> RegisterEntry<K, T> {
         let mut builder = self.builder;
 
         for (tid, v) in self.receivers {
-            for (r, poller) in v {
+            for (r, poller, poller2) in v {
                 let poller = poller(self.item.clone());
 
-                builder.add_recevier((tid, r), poller);
+                builder.add_recevier((tid, r), poller, poller2);
             }
         }
 
@@ -65,39 +69,45 @@ impl<K, T: 'static> RegisterEntry<K, T> {
     }
 }
 
-impl<T: Send + 'static> RegisterEntry<UnsyncEntry, T> {
-    pub fn subscribe<M, R>(mut self, cfg: R::Config) -> Self
+impl<T> RegisterEntry<UnsyncEntry, T> {
+    pub fn subscribe<M, S, R, E>(mut self, queue: u64, cfg: S::Config) -> Self
     where
         T: Send + 'static,
-        M: Message + 'static,
-        R: ReceiverSubscriberBuilder<M, T> + 'static,
+        M: Message,
+        R: Message,
+        E: crate::Error,
+        S: ReceiverSubscriberBuilder<T, M, R, E> + 'static,
     {
-        let (inner, poller) = R::build(cfg).subscribe();
+        let (inner, poller) = S::build(cfg);
 
-        let receiver = Receiver::new(inner);
+        let receiver = Receiver::new::<M, R, E, S>(queue, inner);
+        let poller2 = receiver.start_polling_events::<R, E>();
         self.receivers
             .entry(TypeId::of::<M>())
             .or_insert_with(Vec::new)
-            .push((receiver, poller));
+            .push((receiver, poller, poller2));
 
         self
     }
 }
 
-impl<T: Send + Sync + 'static> RegisterEntry<SyncEntry, T> {
-    pub fn subscribe<M, R>(mut self, cfg: R::Config) -> Self
+impl<T> RegisterEntry<SyncEntry, T> {
+    pub fn subscribe<M, S, R, E>(mut self, queue: u64, cfg: S::Config) -> Self
     where
-        T: Send + 'static,
-        M: Message + 'static,
-        R: ReceiverSubscriberBuilder<M, T> + 'static,
+        T: Send + Sync + 'static,
+        M: Message,
+        R: Message,
+        E: crate::Error,
+        S: ReceiverSubscriberBuilder<T, M, R, E> + 'static,
     {
-        let (inner, poller) = R::build(cfg).subscribe();
+        let (inner, poller) = S::build(cfg);
 
-        let receiver = Receiver::new(inner);
+        let receiver = Receiver::new::<M, R, E, S>(queue, inner);
+        let poller2 = receiver.start_polling_events::<R, E>();
         self.receivers
             .entry(TypeId::of::<M>())
             .or_insert_with(Vec::new)
-            .push((receiver, poller));
+            .push((receiver, poller, poller2));
 
         self
     }
@@ -138,9 +148,11 @@ impl BusBuilder {
         &mut self,
         val: (TypeId, Receiver),
         poller: Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
+        poller2: Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
     ) {
         self.receivers.push(val);
         self.pollings.push(poller);
+        self.pollings.push(poller2);
     }
 
     pub fn build(self) -> (Bus, impl Future<Output = ()>) {
@@ -148,7 +160,7 @@ impl BusBuilder {
             inner: Arc::new(BusInner::new(self.receivers)),
         };
 
-        let mut futs = Vec::with_capacity(self.pollings.len());
+        let mut futs = Vec::with_capacity(self.pollings.len() * 2);
         for poller in self.pollings {
             futs.push(tokio::task::spawn(poller(bus.clone())));
         }
